import * as i0 from '@angular/core';
import { Pipe, Input, Host, Directive, EventEmitter, HostListener, Output, forwardRef, ChangeDetectionStrategy, Component } from '@angular/core';
import * as i2 from '@angular/common';
import { CommonModule } from '@angular/common';
import * as i3 from '@angular/forms';
import { NG_VALUE_ACCESSOR, FormControl, ReactiveFormsModule } from '@angular/forms';
import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

class MultiSelectSearchFilter {
    _lastOptions;
    _searchCache = {};
    _searchCacheInclusive = {};
    _prevSkippedItems = {};
    transform(options, str = '', limit = 0, renderLimit = 0, searchFunction) {
        str = str.toLowerCase();
        // Drop cache because options were updated
        if (options !== this._lastOptions) {
            this._lastOptions = options;
            this._searchCache = {};
            this._searchCacheInclusive = {};
            this._prevSkippedItems = {};
        }
        const filteredOpts = this._searchCache.hasOwnProperty(str)
            ? this._searchCache[str]
            : this._doSearch(options, str, limit, searchFunction);
        const isUnderLimit = options.length <= limit;
        return isUnderLimit
            ? filteredOpts
            : this._limitRenderedItems(filteredOpts, renderLimit);
    }
    _getSubsetOptions(options, prevOptions, prevSearchStr) {
        const prevInclusiveOrIdx = this._searchCacheInclusive[prevSearchStr];
        if (prevInclusiveOrIdx === true) {
            // If have previous results and it was inclusive, do only subsearch
            return prevOptions;
        }
        else if (typeof prevInclusiveOrIdx === 'number') {
            // Or reuse prev results with unchecked ones
            return [...prevOptions, ...options.slice(prevInclusiveOrIdx)];
        }
        return options;
    }
    _doSearch(options, str, limit, searchFunction) {
        const prevStr = str.slice(0, -1);
        const prevResults = this._searchCache[prevStr];
        const prevResultShift = this._prevSkippedItems[prevStr] || 0;
        if (prevResults) {
            options = this._getSubsetOptions(options, prevResults, prevStr);
        }
        const optsLength = options.length;
        const maxFound = limit > 0 ? Math.min(limit, optsLength) : optsLength;
        const regexp = searchFunction(str);
        const filteredOpts = [];
        let i = 0, founded = 0, removedFromPrevResult = 0;
        const doesOptionMatch = (option) => regexp.test(option.name);
        const getChildren = (option) => options.filter(child => child.parentId === option.id);
        const getParent = (option) => options.find(parent => option.parentId === parent.id);
        const foundFn = (item) => { filteredOpts.push(item); founded++; };
        const notFoundFn = prevResults ? () => removedFromPrevResult++ : () => { };
        for (; i < optsLength && founded < maxFound; ++i) {
            const option = options[i];
            const directMatch = doesOptionMatch(option);
            if (directMatch) {
                foundFn(option);
                continue;
            }
            if (typeof option.parentId === 'undefined') {
                const childrenMatch = getChildren(option).some(doesOptionMatch);
                if (childrenMatch) {
                    foundFn(option);
                    continue;
                }
            }
            if (typeof option.parentId !== 'undefined') {
                const parentMatch = doesOptionMatch(getParent(option));
                if (parentMatch) {
                    foundFn(option);
                    continue;
                }
            }
            notFoundFn();
        }
        const totalIterations = i + prevResultShift;
        this._searchCache[str] = filteredOpts;
        this._searchCacheInclusive[str] = i === optsLength || totalIterations;
        this._prevSkippedItems[str] = removedFromPrevResult + prevResultShift;
        return filteredOpts;
    }
    _limitRenderedItems(items, limit) {
        return items.length > limit && limit > 0 ? items.slice(0, limit) : items;
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: MultiSelectSearchFilter, deps: [], target: i0.ɵɵFactoryTarget.Pipe });
    static ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "14.0.0", version: "19.2.13", ngImport: i0, type: MultiSelectSearchFilter, isStandalone: true, name: "searchFilter" });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: MultiSelectSearchFilter, decorators: [{
            type: Pipe,
            args: [{
                    name: 'searchFilter',
                    standalone: true,
                }]
        }] });

class AutofocusDirective {
    elemRef;
    /**
     * Will set focus if set to truthy value or not set at all
     */
    ssAutofocus;
    get element() {
        return this.elemRef.nativeElement;
    }
    constructor(elemRef) {
        this.elemRef = elemRef;
    }
    ngOnInit() {
        this.focus();
    }
    ngOnChanges(changes) {
        const ssAutofocusChange = changes.ssAutofocus;
        if (ssAutofocusChange && !ssAutofocusChange.isFirstChange()) {
            this.focus();
        }
    }
    focus() {
        if (this.ssAutofocus === undefined || this.ssAutofocus) {
            this.element.focus && this.element.focus();
        }
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: AutofocusDirective, deps: [{ token: i0.ElementRef, host: true }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.2.13", type: AutofocusDirective, isStandalone: true, selector: "[ssAutofocus]", inputs: { ssAutofocus: "ssAutofocus" }, usesOnChanges: true, ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: AutofocusDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[ssAutofocus]',
                    standalone: true,
                }]
        }], ctorParameters: () => [{ type: i0.ElementRef, decorators: [{
                    type: Host
                }] }], propDecorators: { ssAutofocus: [{
                type: Input
            }] } });

class OffClickDirective {
    onOffClick = new EventEmitter();
    _clickEvent;
    _touchEvent;
    onClick(event) {
        this._clickEvent = event;
    }
    onTouch(event) {
        this._touchEvent = event;
    }
    onDocumentClick(event) {
        if (event !== this._clickEvent) {
            this.onOffClick.emit(event);
        }
    }
    onDocumentTouch(event) {
        if (event !== this._touchEvent) {
            this.onOffClick.emit(event);
        }
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: OffClickDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.2.13", type: OffClickDirective, isStandalone: true, selector: "[offClick]", outputs: { onOffClick: "offClick" }, host: { listeners: { "click": "onClick($event)", "touchstart": "onTouch($event)", "document:click": "onDocumentClick($event)", "document:touchstart": "onDocumentTouch($event)" } }, ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: OffClickDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[offClick]',
                    standalone: true,
                }]
        }], propDecorators: { onOffClick: [{
                type: Output,
                args: ['offClick']
            }], onClick: [{
                type: HostListener,
                args: ['click', ['$event']]
            }], onTouch: [{
                type: HostListener,
                args: ['touchstart', ['$event']]
            }], onDocumentClick: [{
                type: HostListener,
                args: ['document:click', ['$event']]
            }], onDocumentTouch: [{
                type: HostListener,
                args: ['document:touchstart', ['$event']]
            }] } });

/*
 * Angular 2 Dropdown Multiselect for Bootstrap
 *
 * Simon Lindh
 * https://github.com/softsimon/ngx-bootstrap-multiselect
 */
const MULTISELECT_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(() => MultiselectDropdownComponent),
    multi: true,
};
class MultiselectDropdownComponent {
    element;
    searchFilter;
    cdRef;
    filterControl = new FormControl('');
    options;
    settings;
    texts;
    disabled = false;
    disabledSelection = false;
    searchFunction = this._escapeRegExp;
    selectionLimitReached = new EventEmitter();
    dropdownClosed = new EventEmitter();
    dropdownOpened = new EventEmitter();
    onAdded = new EventEmitter();
    onRemoved = new EventEmitter();
    onLazyLoad = new EventEmitter();
    onFilter = this.filterControl.valueChanges;
    get focusBack() {
        return this.settings.focusBack && this._focusBack;
    }
    clickedOutside() {
        if (!this.isVisible || !this.settings.closeOnClickOutside) {
            return;
        }
        this.isVisible = false;
        this._focusBack = true;
        this.dropdownClosed.emit();
    }
    destroyed$ = new Subject();
    filteredOptions = [];
    lazyLoadOptions = [];
    renderFilteredOptions = [];
    model = [];
    prevModel = [];
    parents;
    title;
    differ;
    numSelected = 0;
    set isVisible(val) {
        this._isVisible = val;
        this._workerDocClicked = val ? false : this._workerDocClicked;
    }
    get isVisible() {
        return this._isVisible;
    }
    renderItems = true;
    checkAllSearchRegister = new Set();
    checkAllStatus = false;
    loadedValueIds = [];
    _focusBack = false;
    focusedItem;
    showDividerAfterCheck = true;
    defaultSettings = {
        closeOnClickOutside: true,
        pullRight: false,
        enableSearch: false,
        searchRenderLimit: 0,
        searchRenderAfter: 1,
        searchMaxLimit: 0,
        searchMaxRenderedItems: 0,
        checkedStyle: 'checkboxes',
        buttonClasses: 'btn btn-primary dropdown-toggle',
        containerClasses: 'dropdown-inline',
        selectionLimit: 0,
        minSelectionLimit: 0,
        closeOnSelect: false,
        autoUnselect: false,
        showCheckAll: false,
        showUncheckAll: false,
        fixedTitle: false,
        dynamicTitleMaxItems: 3,
        maxHeight: '300px',
        isLazyLoad: false,
        stopScrollPropagation: false,
        loadViewDistance: 1,
        selectAddedValues: false,
        ignoreLabels: false,
        maintainSelectionOrderInTitle: false,
        focusBack: true,
    };
    defaultTexts = {
        checkAll: 'Check all',
        uncheckAll: 'Uncheck all',
        checked: 'checked',
        checkedPlural: 'checked',
        searchPlaceholder: 'Search...',
        searchEmptyResult: 'Nothing found...',
        searchNoRenderText: 'Type in search box to see results...',
        defaultTitle: 'Select',
        allSelected: 'All selected',
    };
    get searchLimit() {
        return this.settings.searchRenderLimit;
    }
    get searchRenderAfter() {
        return this.settings.searchRenderAfter;
    }
    get searchLimitApplied() {
        return this.searchLimit > 0 && this.options.length > this.searchLimit;
    }
    _isVisible = false;
    _workerDocClicked = false;
    constructor(element, searchFilter, differs, cdRef) {
        this.element = element;
        this.searchFilter = searchFilter;
        this.cdRef = cdRef;
        this.differ = differs.find([]).create(null);
        this.settings = this.defaultSettings;
        this.texts = this.defaultTexts;
    }
    getItemStyle(option) {
        const style = {};
        if (!option.isLabel) {
            style['cursor'] = 'pointer';
        }
        if (option.disabled) {
            style['cursor'] = 'default';
        }
    }
    getItemStyleSelectionDisabled() {
        if (this.disabledSelection) {
            return { cursor: 'default' };
        }
    }
    ngOnInit() {
        this.title = this.texts.defaultTitle || '';
        this.filterControl.valueChanges.pipe(takeUntil(this.destroyed$)).subscribe(() => {
            this.updateRenderItems();
            if (this.settings.isLazyLoad) {
                this.load();
            }
        });
    }
    ngOnChanges(changes) {
        if (changes['options']) {
            this.options = this.options || [];
            this.parents = this.options
                .filter(option => typeof option.parentId === 'number')
                .map(option => option.parentId);
            this.updateRenderItems();
            if (this.settings.isLazyLoad &&
                this.settings.selectAddedValues &&
                this.loadedValueIds.length === 0) {
                this.loadedValueIds = this.loadedValueIds.concat(changes.options.currentValue.map(value => value.id));
            }
            if (this.settings.isLazyLoad &&
                this.settings.selectAddedValues &&
                changes.options.previousValue) {
                const addedValues = changes.options.currentValue.filter(value => this.loadedValueIds.indexOf(value.id) === -1);
                this.loadedValueIds.concat(addedValues.map(value => value.id));
                if (this.checkAllStatus) {
                    this.addChecks(addedValues);
                }
                else if (this.checkAllSearchRegister.size > 0) {
                    this.checkAllSearchRegister.forEach(searchValue => this.addChecks(this.applyFilters(addedValues, searchValue)));
                }
            }
            if (this.texts) {
                this.updateTitle();
            }
            this.fireModelChange();
        }
        if (changes['settings']) {
            this.settings = { ...this.defaultSettings, ...this.settings };
        }
        if (changes['texts']) {
            this.texts = { ...this.defaultTexts, ...this.texts };
            if (!changes['texts'].isFirstChange()) {
                this.updateTitle();
            }
        }
    }
    ngOnDestroy() {
        this.destroyed$.next();
    }
    updateRenderItems() {
        this.renderItems =
            !this.searchLimitApplied ||
                this.filterControl.value.length >= this.searchRenderAfter;
        this.filteredOptions = this.applyFilters(this.options, this.settings.isLazyLoad ? '' : this.filterControl.value);
        this.renderFilteredOptions = this.renderItems ? this.filteredOptions : [];
        this.focusedItem = undefined;
        // For when 1 parent and 1 child is shown --> no check/uncheck buttons --> no divider
        if ((this.settings.showCheckAll || this.settings.showUncheckAll) && this.renderItems && this.renderFilteredOptions.length > 1) {
            this.showDividerAfterCheck = true;
            if (this.renderFilteredOptions.length === 2) {
                if (this.renderFilteredOptions[0].id === this.renderFilteredOptions[1].parentId) {
                    this.showDividerAfterCheck = false;
                }
            }
        }
    }
    applyFilters(options, value) {
        return this.searchFilter.transform(options, value, this.settings.searchMaxLimit, this.settings.searchMaxRenderedItems, this.searchFunction);
    }
    fireModelChange() {
        if (this.model != this.prevModel) {
            this.prevModel = this.model;
            this.onModelChange(this.model);
            this.onModelTouched();
            this.cdRef.markForCheck();
        }
    }
    onModelChange = (_) => { };
    onModelTouched = () => { };
    writeValue(value) {
        if (value !== undefined && value !== null) {
            this.model = Array.isArray(value) ? value : [value];
            this.ngDoCheck();
        }
        else {
            this.model = [];
        }
    }
    registerOnChange(fn) {
        this.onModelChange = fn;
    }
    registerOnTouched(fn) {
        this.onModelTouched = fn;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
    }
    ngDoCheck() {
        const changes = this.differ.diff(this.model);
        if (changes) {
            this.updateNumSelected();
            this.updateTitle();
        }
    }
    validate(_c) {
        if (this.model && this.model.length) {
            return {
                required: {
                    valid: false,
                },
            };
        }
        if (this.options.filter(o => this.model.indexOf(o.id) && !o.disabled).length === 0) {
            return {
                selection: {
                    valid: false,
                },
            };
        }
        return null;
    }
    registerOnValidatorChange(_fn) {
        throw new Error('Method not implemented.');
    }
    clearSearch(event) {
        this.maybeStopPropagation(event);
        this.filterControl.setValue('');
        this.showDividerAfterCheck = true;
    }
    toggleDropdown(e) {
        if (this.isVisible) {
            this._focusBack = true;
        }
        this.isVisible = !this.isVisible;
        this.isVisible ? this.dropdownOpened.emit() : this.dropdownClosed.emit();
        this.focusedItem = undefined;
    }
    closeDropdown(e) {
        this.isVisible = true;
        this.toggleDropdown(e);
    }
    isSelected(option) {
        return this.model && this.model.indexOf(option.id) > -1;
    }
    setSelected(_event, option) {
        if (option.isLabel) {
            return;
        }
        if (option.disabled) {
            return;
        }
        if (this.disabledSelection) {
            return;
        }
        this.maybeStopPropagation(_event);
        this.maybePreventDefault(_event);
        const index = this.model.indexOf(option.id);
        const isAtSelectionLimit = this.settings.selectionLimit > 0 &&
            this.model.length >= this.settings.selectionLimit;
        const removeItem = (idx, id) => {
            this.model.splice(idx, 1);
            this.onRemoved.emit(id);
            if (this.settings.isLazyLoad &&
                this.lazyLoadOptions.some(val => val.id === id)) {
                this.lazyLoadOptions.splice(this.lazyLoadOptions.indexOf(this.lazyLoadOptions.find(val => val.id === id)), 1);
            }
        };
        if (index > -1) {
            if (this.settings.minSelectionLimit === undefined ||
                this.numSelected > this.settings.minSelectionLimit) {
                removeItem(index, option.id);
            }
            const parentIndex = option.parentId && this.model.indexOf(option.parentId);
            if (parentIndex > -1) {
                removeItem(parentIndex, option.parentId);
            }
            else if (this.parents.indexOf(option.id) > -1) {
                this.options
                    .filter(child => this.model.indexOf(child.id) > -1 &&
                    child.parentId === option.id)
                    .forEach(child => removeItem(this.model.indexOf(child.id), child.id));
            }
        }
        else if (isAtSelectionLimit && !this.settings.autoUnselect) {
            this.selectionLimitReached.emit(this.model.length);
            return;
        }
        else {
            const addItem = (id) => {
                this.model.push(id);
                this.onAdded.emit(id);
                if (this.settings.isLazyLoad &&
                    !this.lazyLoadOptions.some(val => val.id === id)) {
                    this.lazyLoadOptions.push(option);
                }
            };
            addItem(option.id);
            if (!isAtSelectionLimit) {
                if (option.parentId && !this.settings.ignoreLabels) {
                    const children = this.options.filter(child => child.id !== option.id && child.parentId === option.parentId);
                    if (children.every(child => this.model.indexOf(child.id) > -1)) {
                        addItem(option.parentId);
                    }
                }
                else if (this.parents.indexOf(option.id) > -1) {
                    const children = this.options.filter(child => this.model.indexOf(child.id) < 0 && child.parentId === option.id);
                    children.forEach(child => addItem(child.id));
                }
            }
            else {
                removeItem(0, this.model[0]);
            }
        }
        if (this.settings.closeOnSelect) {
            this.toggleDropdown();
        }
        this.model = this.model.slice();
        this.fireModelChange();
    }
    updateNumSelected() {
        this.numSelected =
            this.model.filter(id => this.parents.indexOf(id) < 0).length || 0;
    }
    updateTitle() {
        let numSelectedOptions = this.options.length;
        if (this.settings.ignoreLabels) {
            numSelectedOptions = this.options.filter((option) => !option.isLabel).length;
        }
        if (this.numSelected === 0 || this.settings.fixedTitle) {
            this.title = this.texts ? this.texts.defaultTitle : '';
        }
        else if (this.settings.displayAllSelectedText &&
            this.model.length === numSelectedOptions) {
            this.title = this.texts ? this.texts.allSelected : '';
        }
        else if (this.settings.dynamicTitleMaxItems &&
            this.settings.dynamicTitleMaxItems >= this.numSelected) {
            const useOptions = this.settings.isLazyLoad && this.lazyLoadOptions.length
                ? this.lazyLoadOptions
                : this.options;
            let titleSelections;
            if (this.settings.maintainSelectionOrderInTitle) {
                const optionIds = useOptions.map((selectOption, idx) => selectOption.id);
                titleSelections = this.model
                    .map(selectedId => optionIds.indexOf(selectedId))
                    .filter(optionIndex => optionIndex > -1)
                    .map(optionIndex => useOptions[optionIndex]);
            }
            else {
                titleSelections = useOptions.filter((option) => this.model.indexOf(option.id) > -1);
            }
            this.title = titleSelections.map((option) => option.name).join(', ');
        }
        else {
            this.title =
                this.numSelected +
                    ' ' +
                    (this.numSelected === 1
                        ? this.texts.checked
                        : this.texts.checkedPlural);
        }
        this.cdRef.markForCheck();
    }
    searchFilterApplied() {
        return (this.settings.enableSearch &&
            this.filterControl.value &&
            this.filterControl.value.length > 0);
    }
    addChecks(options) {
        const checkedOptions = options
            .filter((option) => {
            if (!option.disabled &&
                (this.model.indexOf(option.id) === -1 &&
                    !(this.settings.ignoreLabels && option.isLabel))) {
                this.onAdded.emit(option.id);
                return true;
            }
            return false;
        })
            .map((option) => option.id);
        this.model = this.model.concat(checkedOptions);
    }
    checkAll() {
        if (!this.disabledSelection) {
            this.addChecks(!this.searchFilterApplied() ? this.options : this.filteredOptions);
            if (this.settings.isLazyLoad && this.settings.selectAddedValues) {
                if (this.searchFilterApplied() && !this.checkAllStatus) {
                    this.checkAllSearchRegister.add(this.filterControl.value);
                }
                else {
                    this.checkAllSearchRegister.clear();
                    this.checkAllStatus = true;
                }
                this.load();
            }
            this.fireModelChange();
        }
    }
    uncheckAll() {
        if (!this.disabledSelection) {
            const checkedOptions = this.model;
            let unCheckedOptions = !this.searchFilterApplied()
                ? this.model
                : this.filteredOptions.map((option) => option.id);
            // set unchecked options only to the ones that were checked
            unCheckedOptions = checkedOptions.filter(item => unCheckedOptions.indexOf(item) > -1);
            this.model = this.model.filter((id) => {
                if ((unCheckedOptions.indexOf(id) < 0 &&
                    this.settings.minSelectionLimit === undefined) ||
                    unCheckedOptions.indexOf(id) < this.settings.minSelectionLimit) {
                    return true;
                }
                else {
                    this.onRemoved.emit(id);
                    return false;
                }
            });
            if (this.settings.isLazyLoad && this.settings.selectAddedValues) {
                if (this.searchFilterApplied()) {
                    if (this.checkAllSearchRegister.has(this.filterControl.value)) {
                        this.checkAllSearchRegister.delete(this.filterControl.value);
                        this.checkAllSearchRegister.forEach(function (searchTerm) {
                            const filterOptions = this.applyFilters(this.options.filter(option => unCheckedOptions.indexOf(option.id) > -1), searchTerm);
                            this.addChecks(filterOptions);
                        });
                    }
                }
                else {
                    this.checkAllSearchRegister.clear();
                    this.checkAllStatus = false;
                }
                this.load();
            }
            this.fireModelChange();
        }
    }
    preventCheckboxCheck(event, option) {
        if (option.disabled ||
            (this.settings.selectionLimit &&
                !this.settings.autoUnselect &&
                this.model.length >= this.settings.selectionLimit &&
                this.model.indexOf(option.id) === -1 &&
                this.maybePreventDefault(event))) {
            this.maybePreventDefault(event);
        }
    }
    isCheckboxDisabled(option) {
        return this.disabledSelection || option && option.disabled;
    }
    checkScrollPosition(ev) {
        const scrollTop = ev.target.scrollTop;
        const scrollHeight = ev.target.scrollHeight;
        const scrollElementHeight = ev.target.clientHeight;
        const roundingPixel = 1;
        const gutterPixel = 1;
        if (scrollTop >=
            scrollHeight -
                (1 + this.settings.loadViewDistance) * scrollElementHeight -
                roundingPixel -
                gutterPixel) {
            this.load();
        }
    }
    checkScrollPropagation(ev, element) {
        const scrollTop = element.scrollTop;
        const scrollHeight = element.scrollHeight;
        const scrollElementHeight = element.clientHeight;
        if ((ev.deltaY > 0 && scrollTop + scrollElementHeight >= scrollHeight) ||
            (ev.deltaY < 0 && scrollTop <= 0)) {
            ev = ev || window.event;
            this.maybePreventDefault(ev);
            ev.returnValue = false;
        }
    }
    trackById(idx, selectOption) {
        return selectOption.id;
    }
    load() {
        this.onLazyLoad.emit({
            length: this.options.length,
            filter: this.filterControl.value,
            checkAllSearches: this.checkAllSearchRegister,
            checkAllStatus: this.checkAllStatus,
        });
    }
    focusItem(dir, e) {
        if (!this.isVisible) {
            return;
        }
        this.maybePreventDefault(e);
        const idx = this.filteredOptions.indexOf(this.focusedItem);
        if (idx === -1) {
            this.focusedItem = this.filteredOptions[0];
            return;
        }
        const nextIdx = idx + dir;
        const newIdx = nextIdx < 0
            ? this.filteredOptions.length - 1
            : nextIdx % this.filteredOptions.length;
        this.focusedItem = this.filteredOptions[newIdx];
    }
    maybePreventDefault(e) {
        if (e && e.preventDefault) {
            e.preventDefault();
        }
    }
    maybeStopPropagation(e) {
        if (e && e.stopPropagation) {
            e.stopPropagation();
        }
    }
    _escapeRegExp(str) {
        const regExpStr = str.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, '\\$&');
        return new RegExp(regExpStr, 'i');
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: MultiselectDropdownComponent, deps: [{ token: i0.ElementRef }, { token: MultiSelectSearchFilter }, { token: i0.IterableDiffers }, { token: i0.ChangeDetectorRef }], target: i0.ɵɵFactoryTarget.Component });
    static ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "19.2.13", type: MultiselectDropdownComponent, isStandalone: true, selector: "ss-multiselect-dropdown", inputs: { options: "options", settings: "settings", texts: "texts", disabled: "disabled", disabledSelection: "disabledSelection", searchFunction: "searchFunction" }, outputs: { selectionLimitReached: "selectionLimitReached", dropdownClosed: "dropdownClosed", dropdownOpened: "dropdownOpened", onAdded: "onAdded", onRemoved: "onRemoved", onLazyLoad: "onLazyLoad", onFilter: "onFilter" }, providers: [
            MULTISELECT_VALUE_ACCESSOR,
            MultiSelectSearchFilter,
        ], usesOnChanges: true, ngImport: i0, template: "<div class=\"dropdown\" [ngClass]=\"settings.containerClasses\" [class.open]=\"isVisible\" (offClick)=\"clickedOutside()\">\n  <button type=\"button\" class=\"dropdown-toggle\" [ngClass]=\"settings.buttonClasses\" (click)=\"toggleDropdown($event)\"\n    [disabled]=\"disabled\" [ssAutofocus]=\"focusBack\">\n    {{ title }}\n    <span class=\"caret\"></span>\n  </button>\n  <div #scroller *ngIf=\"isVisible\" class=\"dropdown-menu\"\n    [ngClass]=\"{'chunkydropdown-menu': settings.checkedStyle == 'visual' }\"\n    (scroll)=\"settings.isLazyLoad ? checkScrollPosition($event) : null\"\n    (wheel)=\"settings.stopScrollPropagation ? checkScrollPropagation($event, scroller) : null\"\n    [class.pull-right]=\"settings.pullRight\" [class.dropdown-menu-right]=\"settings.pullRight\"\n    [style.max-height]=\"settings.maxHeight\" style=\"display: block; height: auto; overflow-y: auto;\"\n    (keydown.tab)=\"focusItem(1, $event)\" (keydown.shift.tab)=\"focusItem(-1, $event)\">\n    <div class=\"input-group input-group-merge search-container\" *ngIf=\"settings.enableSearch\">\n      <span class=\"input-group-text\" id=\"basic-addon1\">\n        <i class=\"fa fa-search\" aria-hidden=\"true\"></i>\n      </span>\n      <input type=\"text\" class=\"form-control\" [ssAutofocus] [formControl]=\"filterControl\"\n        [placeholder]=\"texts.searchPlaceholder\" class=\"form-control\"\n        style=\"border-top-left-radius: inherit; border-bottom-left-radius: inherit;\"\n        [style.border-top-right-radius]=\"filterControl.value.length > 0 ? 'inherit' : null\"\n        [style.border-bottom-right-radius]=\"filterControl.value.length > 0 ? 'inherit' : null\">\n      <ng-container *ngIf=\"filterControl.value.length>0\">\n        <button class=\"btn btn-outline-secondary btn-color-onhover\" type=\"button\" (click)=\"clearSearch($event)\"\n          style=\"display: flex; align-items: center;\">\n          <i class=\"fa fa-times\"></i>\n        </button>\n      </ng-container>\n    </div>\n    <div class=\"dropdown-divider\" *ngIf=\"isVisible\"></div>\n    <ng-container\n      *ngIf=\"settings.showCheckAll && !disabledSelection && renderItems && renderFilteredOptions.length > 1\">\n      <ng-container *ngIf=\"renderFilteredOptions.length === 2; else displayCheckAll\">\n        <ng-container *ngIf=\"renderFilteredOptions[0]?.id !== renderFilteredOptions[1]?.parentId\">\n          <ng-container *ngTemplateOutlet=\"displayCheckAll\"></ng-container>\n        </ng-container>\n      </ng-container>\n    </ng-container>\n    <ng-template #displayCheckAll>\n      <a role=\"menuitem\" href=\"javascript:;\" tabindex=\"-1\" class=\"dropdown-item check-control check-control-check\"\n        (click)=\"checkAll()\">\n        <span style=\"width: 16px;\"><span\n            [ngClass]=\"{'glyphicon glyphicon-ok': settings.checkedStyle !== 'fontawesome','fa fa-check': settings.checkedStyle === 'fontawesome'}\"></span></span>\n        {{ texts.checkAll }}\n      </a>\n    </ng-template>\n    <ng-container\n      *ngIf=\"settings.showUncheckAll && !disabledSelection && renderItems && renderFilteredOptions.length > 1\">\n      <ng-container *ngIf=\"renderFilteredOptions.length === 2; else displayUncheckAll\">\n        <ng-container *ngIf=\"renderFilteredOptions[0]?.id !== renderFilteredOptions[1]?.parentId\">\n          <ng-container *ngTemplateOutlet=\"displayUncheckAll\"></ng-container>\n        </ng-container>\n      </ng-container>\n    </ng-container>\n    <ng-template #displayUncheckAll>\n      <a role=\"menuitem\" href=\"javascript:;\" tabindex=\"-1\" class=\"dropdown-item check-control check-control-uncheck\"\n        (click)=\"uncheckAll()\">\n        <span style=\"width: 16px;\"><span\n            [ngClass]=\"{'glyphicon glyphicon-remove': settings.checkedStyle !== 'fontawesome','fa fa-times': settings.checkedStyle === 'fontawesome'}\"></span></span>\n        {{ texts.uncheckAll }}\n      </a>\n    </ng-template>\n    <div\n      *ngIf=\"(settings.showCheckAll || settings.showUncheckAll) && (renderItems && renderFilteredOptions.length > 1) && showDividerAfterCheck\"\n      class=\"dropdown-divider\"></div>\n    <a *ngIf=\"!renderItems\" href=\"javascript:;\" class=\"dropdown-item empty\">{{ texts.searchNoRenderText }}</a>\n    <a *ngIf=\"renderItems && !renderFilteredOptions.length\" href=\"javascript:;\" class=\"dropdown-item empty\">{{\n      texts.searchEmptyResult }}</a>\n    <a class=\"dropdown-item\" href=\"javascript:;\"\n      *ngFor=\"let option of renderFilteredOptions; trackBy: trackById; let i = index;\"\n      [class.active]=\"isSelected(option)\" [ngStyle]=\"getItemStyle(option)\" [ngClass]=\"option.classes\"\n      [class.dropdown-header]=\"option.isLabel\" [ssAutofocus]=\"option === focusedItem\" tabindex=\"-1\"\n      (click)=\"setSelected($event, option)\" (keydown.space)=\"setSelected($event, option)\"\n      (keydown.enter)=\"setSelected($event, option)\">\n      <span *ngIf=\"!option.isLabel; else label\" role=\"menuitem\" tabindex=\"-1\"\n        [ngStyle]=\"getItemStyleSelectionDisabled()\"\n        [style.padding-left]=\"this.parents.length>0&&this.parents.indexOf(option.id)<0 && '30px'\">\n        <ng-container [ngSwitch]=\"settings.checkedStyle\">\n          <input *ngSwitchCase=\"'checkboxes'\" type=\"checkbox\" [checked]=\"isSelected(option)\"\n            (click)=\"preventCheckboxCheck($event, option)\" [disabled]=\"isCheckboxDisabled(option)\"\n            [ngStyle]=\"getItemStyleSelectionDisabled()\" />\n          <span *ngSwitchCase=\"'glyphicon'\" style=\"width: 16px;\" class=\"glyphicon\"\n            [class.glyphicon-ok]=\"isSelected(option)\" [class.glyphicon-lock]=\"isCheckboxDisabled(option)\"></span>\n          <span *ngSwitchCase=\"'fontawesome'\" style=\"width: 16px;display: inline-block;\">\n            <span *ngIf=\"isSelected(option)\"><i class=\"fa fa-check\" aria-hidden=\"true\"></i></span>\n            <span *ngIf=\"isCheckboxDisabled(option)\"><i class=\"fa fa-lock\" aria-hidden=\"true\"></i></span>\n          </span>\n          <span *ngSwitchCase=\"'visual'\"\n            style=\"display:block;float:left; border-radius: 0.2em; border: 0.1em solid rgba(44, 44, 44, 0.63);background:rgba(0, 0, 0, 0.1);width: 5.5em;\">\n            <div class=\"slider\" [ngClass]=\"{'slideron': isSelected(option)}\">\n              <img *ngIf=\"option.image != null\" [src]=\"option.image\"\n                style=\"height: 100%; width: 100%; object-fit: contain\" />\n              <div *ngIf=\"option.image == null\"\n                style=\"height: 100%; width: 100%;text-align: center; display: table; background-color:rgba(0, 0, 0, 0.74)\">\n                <div class=\"content_wrapper\">\n                  <span style=\"font-size:3em;color:white\" class=\"glyphicon glyphicon-eye-close\"></span>\n                </div>\n              </div>\n            </div>\n          </span>\n        </ng-container>\n        <span [ngClass]=\"{'chunkyrow': settings.checkedStyle == 'visual' }\"\n          [class.disabled]=\"isCheckboxDisabled(option)\" [ngClass]=\"settings.itemClasses\"\n          [style.font-weight]=\"this.parents.indexOf(option.id)>=0?'bold':'normal'\">\n          {{ option.name }}\n        </span>\n      </span>\n      <ng-template #label>\n        <span [class.disabled]=\"isCheckboxDisabled(option)\">{{ option.name }}</span>\n      </ng-template>\n    </a>\n  </div>\n</div>", styles: ["a{outline:none!important}.dropdown-inline{display:inline-block}.dropdown-toggle .caret{margin-left:4px;white-space:nowrap;display:inline-block}.chunkydropdown-menu{min-width:20em}.chunkyrow{line-height:2;margin-left:1em;font-size:2em}.slider{width:3.8em;height:3.8em;display:block;-webkit-transition:all .125s linear;-moz-transition:all .125s linear;-o-transition:all .125s linear;transition:all .125s linear;margin-left:.125em;margin-top:auto}.slideron{margin-left:1.35em}.content_wrapper{display:table-cell;vertical-align:middle}.search-container{padding:0 5px 5px}\n"], dependencies: [{ kind: "ngmodule", type: CommonModule }, { kind: "directive", type: i2.NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }, { kind: "directive", type: i2.NgForOf, selector: "[ngFor][ngForOf]", inputs: ["ngForOf", "ngForTrackBy", "ngForTemplate"] }, { kind: "directive", type: i2.NgIf, selector: "[ngIf]", inputs: ["ngIf", "ngIfThen", "ngIfElse"] }, { kind: "directive", type: i2.NgTemplateOutlet, selector: "[ngTemplateOutlet]", inputs: ["ngTemplateOutletContext", "ngTemplateOutlet", "ngTemplateOutletInjector"] }, { kind: "directive", type: i2.NgStyle, selector: "[ngStyle]", inputs: ["ngStyle"] }, { kind: "directive", type: i2.NgSwitch, selector: "[ngSwitch]", inputs: ["ngSwitch"] }, { kind: "directive", type: i2.NgSwitchCase, selector: "[ngSwitchCase]", inputs: ["ngSwitchCase"] }, { kind: "ngmodule", type: ReactiveFormsModule }, { kind: "directive", type: i3.DefaultValueAccessor, selector: "input:not([type=checkbox])[formControlName],textarea[formControlName],input:not([type=checkbox])[formControl],textarea[formControl],input:not([type=checkbox])[ngModel],textarea[ngModel],[ngDefaultControl]" }, { kind: "directive", type: i3.NgControlStatus, selector: "[formControlName],[ngModel],[formControl]" }, { kind: "directive", type: i3.FormControlDirective, selector: "[formControl]", inputs: ["formControl", "disabled", "ngModel"], outputs: ["ngModelChange"], exportAs: ["ngForm"] }, { kind: "directive", type: AutofocusDirective, selector: "[ssAutofocus]", inputs: ["ssAutofocus"] }, { kind: "directive", type: OffClickDirective, selector: "[offClick]", outputs: ["offClick"] }], changeDetection: i0.ChangeDetectionStrategy.OnPush });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.13", ngImport: i0, type: MultiselectDropdownComponent, decorators: [{
            type: Component,
            args: [{ selector: 'ss-multiselect-dropdown', imports: [
                        CommonModule,
                        ReactiveFormsModule,
                        AutofocusDirective,
                        OffClickDirective,
                    ], providers: [
                        MULTISELECT_VALUE_ACCESSOR,
                        MultiSelectSearchFilter,
                    ], changeDetection: ChangeDetectionStrategy.OnPush, template: "<div class=\"dropdown\" [ngClass]=\"settings.containerClasses\" [class.open]=\"isVisible\" (offClick)=\"clickedOutside()\">\n  <button type=\"button\" class=\"dropdown-toggle\" [ngClass]=\"settings.buttonClasses\" (click)=\"toggleDropdown($event)\"\n    [disabled]=\"disabled\" [ssAutofocus]=\"focusBack\">\n    {{ title }}\n    <span class=\"caret\"></span>\n  </button>\n  <div #scroller *ngIf=\"isVisible\" class=\"dropdown-menu\"\n    [ngClass]=\"{'chunkydropdown-menu': settings.checkedStyle == 'visual' }\"\n    (scroll)=\"settings.isLazyLoad ? checkScrollPosition($event) : null\"\n    (wheel)=\"settings.stopScrollPropagation ? checkScrollPropagation($event, scroller) : null\"\n    [class.pull-right]=\"settings.pullRight\" [class.dropdown-menu-right]=\"settings.pullRight\"\n    [style.max-height]=\"settings.maxHeight\" style=\"display: block; height: auto; overflow-y: auto;\"\n    (keydown.tab)=\"focusItem(1, $event)\" (keydown.shift.tab)=\"focusItem(-1, $event)\">\n    <div class=\"input-group input-group-merge search-container\" *ngIf=\"settings.enableSearch\">\n      <span class=\"input-group-text\" id=\"basic-addon1\">\n        <i class=\"fa fa-search\" aria-hidden=\"true\"></i>\n      </span>\n      <input type=\"text\" class=\"form-control\" [ssAutofocus] [formControl]=\"filterControl\"\n        [placeholder]=\"texts.searchPlaceholder\" class=\"form-control\"\n        style=\"border-top-left-radius: inherit; border-bottom-left-radius: inherit;\"\n        [style.border-top-right-radius]=\"filterControl.value.length > 0 ? 'inherit' : null\"\n        [style.border-bottom-right-radius]=\"filterControl.value.length > 0 ? 'inherit' : null\">\n      <ng-container *ngIf=\"filterControl.value.length>0\">\n        <button class=\"btn btn-outline-secondary btn-color-onhover\" type=\"button\" (click)=\"clearSearch($event)\"\n          style=\"display: flex; align-items: center;\">\n          <i class=\"fa fa-times\"></i>\n        </button>\n      </ng-container>\n    </div>\n    <div class=\"dropdown-divider\" *ngIf=\"isVisible\"></div>\n    <ng-container\n      *ngIf=\"settings.showCheckAll && !disabledSelection && renderItems && renderFilteredOptions.length > 1\">\n      <ng-container *ngIf=\"renderFilteredOptions.length === 2; else displayCheckAll\">\n        <ng-container *ngIf=\"renderFilteredOptions[0]?.id !== renderFilteredOptions[1]?.parentId\">\n          <ng-container *ngTemplateOutlet=\"displayCheckAll\"></ng-container>\n        </ng-container>\n      </ng-container>\n    </ng-container>\n    <ng-template #displayCheckAll>\n      <a role=\"menuitem\" href=\"javascript:;\" tabindex=\"-1\" class=\"dropdown-item check-control check-control-check\"\n        (click)=\"checkAll()\">\n        <span style=\"width: 16px;\"><span\n            [ngClass]=\"{'glyphicon glyphicon-ok': settings.checkedStyle !== 'fontawesome','fa fa-check': settings.checkedStyle === 'fontawesome'}\"></span></span>\n        {{ texts.checkAll }}\n      </a>\n    </ng-template>\n    <ng-container\n      *ngIf=\"settings.showUncheckAll && !disabledSelection && renderItems && renderFilteredOptions.length > 1\">\n      <ng-container *ngIf=\"renderFilteredOptions.length === 2; else displayUncheckAll\">\n        <ng-container *ngIf=\"renderFilteredOptions[0]?.id !== renderFilteredOptions[1]?.parentId\">\n          <ng-container *ngTemplateOutlet=\"displayUncheckAll\"></ng-container>\n        </ng-container>\n      </ng-container>\n    </ng-container>\n    <ng-template #displayUncheckAll>\n      <a role=\"menuitem\" href=\"javascript:;\" tabindex=\"-1\" class=\"dropdown-item check-control check-control-uncheck\"\n        (click)=\"uncheckAll()\">\n        <span style=\"width: 16px;\"><span\n            [ngClass]=\"{'glyphicon glyphicon-remove': settings.checkedStyle !== 'fontawesome','fa fa-times': settings.checkedStyle === 'fontawesome'}\"></span></span>\n        {{ texts.uncheckAll }}\n      </a>\n    </ng-template>\n    <div\n      *ngIf=\"(settings.showCheckAll || settings.showUncheckAll) && (renderItems && renderFilteredOptions.length > 1) && showDividerAfterCheck\"\n      class=\"dropdown-divider\"></div>\n    <a *ngIf=\"!renderItems\" href=\"javascript:;\" class=\"dropdown-item empty\">{{ texts.searchNoRenderText }}</a>\n    <a *ngIf=\"renderItems && !renderFilteredOptions.length\" href=\"javascript:;\" class=\"dropdown-item empty\">{{\n      texts.searchEmptyResult }}</a>\n    <a class=\"dropdown-item\" href=\"javascript:;\"\n      *ngFor=\"let option of renderFilteredOptions; trackBy: trackById; let i = index;\"\n      [class.active]=\"isSelected(option)\" [ngStyle]=\"getItemStyle(option)\" [ngClass]=\"option.classes\"\n      [class.dropdown-header]=\"option.isLabel\" [ssAutofocus]=\"option === focusedItem\" tabindex=\"-1\"\n      (click)=\"setSelected($event, option)\" (keydown.space)=\"setSelected($event, option)\"\n      (keydown.enter)=\"setSelected($event, option)\">\n      <span *ngIf=\"!option.isLabel; else label\" role=\"menuitem\" tabindex=\"-1\"\n        [ngStyle]=\"getItemStyleSelectionDisabled()\"\n        [style.padding-left]=\"this.parents.length>0&&this.parents.indexOf(option.id)<0 && '30px'\">\n        <ng-container [ngSwitch]=\"settings.checkedStyle\">\n          <input *ngSwitchCase=\"'checkboxes'\" type=\"checkbox\" [checked]=\"isSelected(option)\"\n            (click)=\"preventCheckboxCheck($event, option)\" [disabled]=\"isCheckboxDisabled(option)\"\n            [ngStyle]=\"getItemStyleSelectionDisabled()\" />\n          <span *ngSwitchCase=\"'glyphicon'\" style=\"width: 16px;\" class=\"glyphicon\"\n            [class.glyphicon-ok]=\"isSelected(option)\" [class.glyphicon-lock]=\"isCheckboxDisabled(option)\"></span>\n          <span *ngSwitchCase=\"'fontawesome'\" style=\"width: 16px;display: inline-block;\">\n            <span *ngIf=\"isSelected(option)\"><i class=\"fa fa-check\" aria-hidden=\"true\"></i></span>\n            <span *ngIf=\"isCheckboxDisabled(option)\"><i class=\"fa fa-lock\" aria-hidden=\"true\"></i></span>\n          </span>\n          <span *ngSwitchCase=\"'visual'\"\n            style=\"display:block;float:left; border-radius: 0.2em; border: 0.1em solid rgba(44, 44, 44, 0.63);background:rgba(0, 0, 0, 0.1);width: 5.5em;\">\n            <div class=\"slider\" [ngClass]=\"{'slideron': isSelected(option)}\">\n              <img *ngIf=\"option.image != null\" [src]=\"option.image\"\n                style=\"height: 100%; width: 100%; object-fit: contain\" />\n              <div *ngIf=\"option.image == null\"\n                style=\"height: 100%; width: 100%;text-align: center; display: table; background-color:rgba(0, 0, 0, 0.74)\">\n                <div class=\"content_wrapper\">\n                  <span style=\"font-size:3em;color:white\" class=\"glyphicon glyphicon-eye-close\"></span>\n                </div>\n              </div>\n            </div>\n          </span>\n        </ng-container>\n        <span [ngClass]=\"{'chunkyrow': settings.checkedStyle == 'visual' }\"\n          [class.disabled]=\"isCheckboxDisabled(option)\" [ngClass]=\"settings.itemClasses\"\n          [style.font-weight]=\"this.parents.indexOf(option.id)>=0?'bold':'normal'\">\n          {{ option.name }}\n        </span>\n      </span>\n      <ng-template #label>\n        <span [class.disabled]=\"isCheckboxDisabled(option)\">{{ option.name }}</span>\n      </ng-template>\n    </a>\n  </div>\n</div>", styles: ["a{outline:none!important}.dropdown-inline{display:inline-block}.dropdown-toggle .caret{margin-left:4px;white-space:nowrap;display:inline-block}.chunkydropdown-menu{min-width:20em}.chunkyrow{line-height:2;margin-left:1em;font-size:2em}.slider{width:3.8em;height:3.8em;display:block;-webkit-transition:all .125s linear;-moz-transition:all .125s linear;-o-transition:all .125s linear;transition:all .125s linear;margin-left:.125em;margin-top:auto}.slideron{margin-left:1.35em}.content_wrapper{display:table-cell;vertical-align:middle}.search-container{padding:0 5px 5px}\n"] }]
        }], ctorParameters: () => [{ type: i0.ElementRef }, { type: MultiSelectSearchFilter }, { type: i0.IterableDiffers }, { type: i0.ChangeDetectorRef }], propDecorators: { options: [{
                type: Input
            }], settings: [{
                type: Input
            }], texts: [{
                type: Input
            }], disabled: [{
                type: Input
            }], disabledSelection: [{
                type: Input
            }], searchFunction: [{
                type: Input
            }], selectionLimitReached: [{
                type: Output
            }], dropdownClosed: [{
                type: Output
            }], dropdownOpened: [{
                type: Output
            }], onAdded: [{
                type: Output
            }], onRemoved: [{
                type: Output
            }], onLazyLoad: [{
                type: Output
            }], onFilter: [{
                type: Output
            }] } });

/**
 * Generated bundle index. Do not edit.
 */

export { MultiSelectSearchFilter, MultiselectDropdownComponent };
//# sourceMappingURL=ngx-bootstrap-multiselect-updated.mjs.map
